#include "wall.h"
#include <cstring>

const int D = 21;
const int N = 1 << D;
const int INF = 100500;

int segNear(int x, int l, int r)
{
	if (x < l)
		return l;
	if (x > r)
		return r;
	return x;
}

class SegTree
{
private:
	int treeLower[N * 2 - 1], treeUpper[N * 2 - 1];

	void updateChilds(int e)
	{
		int e1 = e * 2 + 1, e2 = e * 2 + 2;
		treeLower[e1] = segNear(treeLower[e1], treeLower[e], treeUpper[e]);
		treeUpper[e1] = segNear(treeUpper[e1], treeLower[e], treeUpper[e]);
		treeLower[e2] = segNear(treeLower[e2], treeLower[e], treeUpper[e]);
		treeUpper[e2] = segNear(treeUpper[e2], treeLower[e], treeUpper[e]);
		treeLower[e] = -INF;
		treeUpper[e] = INF;
	}

public:
	SegTree()
	{
		for (int i = 0; i < N; ++i)
			treeLower[i + N - 1] = treeUpper[i + N - 1] = 0;
		for (int i = 0; i < N - 1; ++i)
			treeLower[i] = -(treeUpper[i] = INF);
	}

	void set(int l, int r, int low, int up, int e = 0, int cl = 0, int cr = N - 1)
	{
		if (l > cr || r < cl)
			return;
		if (l <= cl && cr <= r)
		{
			treeLower[e] = segNear(treeLower[e], low, up);
			treeUpper[e] = segNear(treeUpper[e], low, up);
			return;
		}
		updateChilds(e);
		set(l, r, low, up, e * 2 + 1, cl, (cl + cr) / 2);
		set(l, r, low, up, e * 2 + 2, (cl + cr) / 2 + 1, cr);
	}

	void getAnswer(int * answer, int n)
	{
		for (int i = 0; i < N - 1; ++i)
			updateChilds(i);
		memcpy(answer, treeLower + N - 1, n * sizeof(int));
	}
};

void buildWall(int n, int k, int * op, int * left, int * right, int * height, int * finalHeight)
{
	static SegTree st;
	for (int i = 0; i < k; ++i)
		switch (op[i])
		{
		case 1:
			st.set(left[i], right[i], height[i], INF);
			break;
		case 2:
			st.set(left[i], right[i], -INF, height[i]);
			break;
		};
	st.getAnswer(finalHeight, n);
}