#include "grader.h"
#include <cstring>

static const int B = 19;

class BigInt
{
protected:
	unsigned d[B];
public:
	BigInt()
	{
		memset(d, 0, sizeof(d));
	}

	BigInt(int n, int a[])
	{
		memset(d, 0, sizeof(d));
		unsigned char * dc = (unsigned char *)d;
		for (int i = 0; i < n; ++i)
			dc[i] = (unsigned char)(a[i]);
	}

	BigInt(unsigned a)
	{
		memset(d, 0, sizeof(d));
		d[0] = a;
	}

	BigInt & operator += (const BigInt & b)
	{
		unsigned next = 0;
		for (int i = 0; i < B; ++i)
		{
			unsigned was = d[i];
			d[i] += b.d[i] + next;
			next = (d[i] < was) ? 1 : 0;
		}
		return *this;
	}

	BigInt & operator -= (const BigInt & b)
	{
		unsigned next = 1;
		for (int i = 0; i < B; ++i)
		{
			unsigned was = d[i];
			d[i] += (~b.d[i]) + next;
			next = (d[i] <= was) ? 1 : 0;
		}
		return *this;
	}

	bool operator < (const BigInt & b) const
	{
		for (int i = B - 1; i >= 0; --i)
			if (d[i] < b.d[i])
				return true;
			else if (d[i] > b.d[i])
				return false;
		return false;
	}

	bool operator > (const BigInt & b) const
	{
		return b < *this;
	}

	bool operator <= (const BigInt & b) const
	{
		return !(b < *this);
	}

	bool operator >= (const BigInt & b) const
	{
		return !(*this < b);
	}

	bool operator == (const BigInt & b) const
	{
		return memcmp(d, b.d, sizeof(d)) == 0;
	}

	unsigned char getByte(int k) const
	{
		return (d[k / 4] >> (8 * (k % 4))) & 255;
	}
};

static BigInt comb[601][601];
static void computeComb()
{
	for (int n = 0; n < 601; ++n)
		for (int k = 0; k <= n; ++k)
		{
			if (k == 0 || k == n)
			{
				comb[n][k] = 1;
				continue;
			}
			comb[n][k] = comb[n - 1][k - 1];
			comb[n][k] += comb[n - 1][k];
		}
}

static void getCombinationByNumber(int n, int k, const BigInt & num, bool a[])
{
	BigInt x = num;
	for (int i = 0; i < n; ++i)
	{
		if (x < comb[n - i - 1][k])
			a[i] = false;
		else
		{
			a[i] = true;
			x -= comb[n - i - 1][k];
			--k;
		}
	}
	/*std::cerr << num << " -> ";
	for (int i = 0; i < n; ++i)
		std::cerr << (a[i] ? '#' : '.');
	std::cerr << "\n";*/
}

static BigInt getCombinationNumber(int n, int k, bool a[])
{
	BigInt answer = 0;
	for (int i = 0; i < n; ++i)
		if (a[i])
			answer += comb[n - i - 1][k--];
	/*for (int i = 0; i < n; ++i)
		std::cerr << (a[i] ? '#' : '.');
	std::cerr << " -> " << answer << "\n";*/
	return answer;
}

void encode(int n, int a[])
{
	computeComb();
	BigInt x(n, a);
	static bool s[600];
	getCombinationByNumber(n * 5 + 255, 255, x, s);
	/*for (int i = 0; i < n * 5 + 255; ++i)
		std::cerr << (s[i] ? '#' : '.');
	std::cerr << "\n";*/
	int prev = -1, cp = 0;
	for (int i = 0; i < n * 5 + 255; ++i)
		if (s[i])
		{
			for (int j = 0; j < i - prev - 1; ++j)
				send(cp);
			prev = i;
			++cp;
		}
	for (int j = 0; j < n * 5 + 254 - prev; ++j)
		send(cp);
}

void decode(int n, int l, int p[])
{
	computeComb();
	static bool s[600];
	memset(s, 0, sizeof(s));
	static int count[256];
	for (int i = 0; i < 256; ++i)
		count[i] = 1;
	for (int i = 0; i < l; ++i)
		++count[p[i]];
	int sum = -1;
	for (int i = 0; i < 255; ++i)
		s[sum += count[i]] = true;
	BigInt answer = getCombinationNumber(n * 5 + 255, 255, s);
	/*for (int i = 0; i < n * 5 + 255; ++i)
		std::cerr << (s[i] ? '#' : '.');
	std::cerr << "\n";*/
	for (int i = 0; i < n; ++i)
		output(answer.getByte(i));
}