#include <vector>
#include <algorithm>
#include <cstdio>
#include <ctime>

const int N = 100500;

struct Node
{
	int l, r;
	int maxLine, maxForce, maxForce1, maxLinePos;
} nodes[N + N];
int nodesCount;

struct Treap
{
	Treap * l, * r;
	int val, prior, size;
	Treap(int nval, Treap * nl = NULL, Treap * nr = NULL);
	~Treap()
	{
		delete l;
		delete r;
	}
};

void print(Treap * t, int q = 0)
{
	for (int i = 0; i < q; ++i)
		printf(" ");
	if (t == NULL)
		printf("NULL\n");
	else
	{
		printf("val=%d,size=%d\n", t->val, t->size);
		print(t->l, q + 1);
		print(t->r, q + 1);
	}
}

int getSize(Treap * t)
{
	if (t == NULL)
		return 0;
	return t->size;
}

Treap::Treap(int nval, Treap * nl, Treap * nr) :
	val(nval), l(nl), r(nr), prior(rand()), size(1 + getSize(nl) + getSize(nr))
{
}

Treap * merge(Treap * l, Treap * r)
{
	if (l == NULL)
		return r;
	if (r == NULL)
		return l;
	if (l->prior > r->prior)
	{
		l->r = merge(l->r, r);
		l->size = 1 + getSize(l->l) + getSize(l->r);
		return l;
	} else {
		r->l = merge(l, r->l);
		r->size = 1 + getSize(r->l) + getSize(r->r);
		return r;
	}
}

void split(Treap * t, int x, Treap *& l, Treap *& r)
{
	if (t == NULL)
	{
		l = r = t;
		return;
	}
	Treap * nl, * nr;
	if (x <= getSize(t->l))
	{
		split(t->l, x, nl, t->l);
		nr = t;
	} else {
		split(t->r, x - getSize(t->l) - 1, t->r, nr);
		nl = t;
	}
	if (nl != NULL)
		nl->size = 1 + getSize(nl->l) + getSize(nl->r);
	if (nr != NULL)
		nr->size = 1 + getSize(nr->l) + getSize(nr->r);
	l = nl;
	r = nr;
}

Treap * t = NULL;

int GetBestPosition(int n, int c, int r, int * k, int * s, int * e)
{
	srand(time(NULL));
	int answer = 0, answerLine = 0;
	for (int i = 0; i < n; ++i)
	{
		nodes[nodesCount].l = nodes[nodesCount].r = i;
		nodes[nodesCount].maxLine = 0;
		nodes[nodesCount].maxLinePos = i;
		nodes[nodesCount].maxForce = (i == n - 1) ? -1 : k[i];
		nodes[nodesCount].maxForce1 = -1;
		t = merge(t, new Treap(nodesCount));
		++nodesCount;
	}
	for (int i = 0; i < c; ++i)
	{
		Treap * t1, * t2, * t3;
		split(t, e[i] + 1, t2, t3);
		split(t2, s[i], t1, t2);
		nodes[nodesCount].l = s[i];
		nodes[nodesCount].r = e[i];

		nodes[nodesCount].maxLine = 0;
		while (t2 != NULL)
		{
			Treap * nd;
			split(t2, 1, nd, t2);
			int node = nd->val;
			if (nodes[node].maxLine + 1 > nodes[nodesCount].maxLine)
			{
				nodes[nodesCount].maxLine = nodes[node].maxLine + 1;
				nodes[nodesCount].maxLinePos = nodes[node].maxLinePos;
			}
			nodes[nodesCount].maxForce = std::max(nodes[nodesCount].maxForce, nodes[node].maxForce);
			nodes[nodesCount].maxForce1 = std::max(nodes[nodesCount].maxForce1, (getSize(t2) == 0) ? nodes[node].maxForce1 : nodes[node].maxForce);
		}
		t = merge(merge(t1, new Treap(nodesCount)), t3);
		if (nodes[nodesCount].maxForce1 < r && answerLine < nodes[nodesCount].maxLine)
		{
			answerLine = nodes[nodesCount].maxLine;
			answer = nodes[nodesCount].maxLinePos;
		}
		++nodesCount;
	}
	return answer;
}