#include <cstdio>
#include <cmath>
#include <algorithm>

const int N = 500;

const int COLOR_R[8] = {0,255,  0,  0,255,255,  0,255};
const int COLOR_G[8] = {0,  0,255,  0,255,  0,255,255};
const int COLOR_B[8] = {0,  0,  0,255,  0,255,255,255};
const int DX[4] = {0,1,-1, 0};
const int DY[4] = {1,0, 0,-1};
const int COUNT_MANY = 20;
const int COUNT_NOT_SO_MANY = 12;
const double GREEN_PART = 0.3;

int rgbTo8(int r, int g, int b)
{
	int a = 0, f = abs(r - COLOR_R[0]) + abs(g - COLOR_G[0]) + abs(b - COLOR_B[0]);
	for (int i = 1; i < 8; ++i)
	{
		int nf = abs(r - COLOR_R[i]) + abs(g - COLOR_G[i]) + abs(b - COLOR_B[i]);
		if (nf < f)
		{
			f = nf;
			a = i;
		}
	}
	return a;
}

static int p[500][500];
int w, h;
int minSize;

bool visited[500][500];

bool inField(int x, int y)
{
	return x >= 0 && y >= 0 && x < h && y < w;
}

int dfs(int x, int y)
{
	if (visited[x][y])
		return 0;
	int a = 1;
	for (int i = 0; i < 4; ++i)
	{
		int nx = x + DX[i];
		int ny = y + DY[i];
		if (inField(nx, ny) && p[x][y] == p[nx][ny])
			a += dfs(nx, ny);
	}
	return a;
}

int style(int n, int m, int r[N][N], int g[N][N], int b[N][N])
{
	w = m;
	h = n;
	minSize = std::max(w, h);
	for (int i = 0; i < h; ++i)
		for (int j = 0; j < w; ++j)
			p[i][j] = rgbTo8(r[i][j], g[i][j], b[i][j]);
	int count = 0;
	for (int i = 0; i < h; ++i)
		for (int j = 0; j < w; ++j)
			if (dfs(i, j) >= minSize)
				++count;
	if (count <= COUNT_NOT_SO_MANY)
		return 4;
	if (count <= COUNT_MANY)
		return 1;
	int gc = 0;
	for (int i = 0; i < h; ++i)
		for (int j = 0; j < w; ++j)
			if (p[i][j] == 2)
				++gc;
	if (gc > (h * w * GREEN_PART))
		return 2;
	return 3;
}
